@@ -285,6 +285,7 @@ static void file_load_cb (GtkWidget *button, PictMenuData *menudata);
 static void file_save_cb (GtkWidget *button, PictMenuData *menudata);
 static void copy_cb (GtkWidget *button, PictMenuData *menudata);
 
+/* Commented out because it's not used:
 static void
 menudata_free (PictMenuData *menudata)
 {
@@ -294,6 +295,7 @@ menudata_free (PictMenuData *menudata)
 	}
 	g_free (menudata);
 }
+*/
 
 void
 common_pict_create_menu (PictMenu *pictmenu, GtkWidget *attach_to, PictBinData *bindata, PictOptions *options,
@@ -457,7 +459,7 @@ file_save_cb (GtkWidget *button, PictMenuData *menudata)
 
 	/* determine writable formats */
 	expander = gtk_expander_new (_("Image format"));
-	hbox = gtk_hbox_new (FALSE, 0);
+	hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0);
 	gtk_container_add (GTK_CONTAINER (expander), hbox);
 
 	label = gtk_label_new (_("Format image as:"));
@@ -544,7 +546,7 @@ file_save_cb (GtkWidget *button, PictMenuData *menudata)
 }
 
 static void
-copy_cb (GtkWidget *button, PictMenuData *menudata)
+copy_cb (G_GNUC_UNUSED GtkWidget *button, PictMenuData *menudata)
 {
 	GtkClipboard *cp;
 	cp = gtk_clipboard_get (gdk_atom_intern_static_string ("CLIPBOARD"));
