@@ -1,23 +1,25 @@
 /*
- * Copyright (C) 2010 Vivien Malerba <malerba@gnome-db.org>
- *               1997-2000 GTK+ Team and others.
+ * Copyright (C) 2010 David King <davidk@openismus.com>
+ * Copyright (C) 2010 - 2011 Vivien Malerba <malerba@gnome-db.org>
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 #include "widget-embedder.h"
+#include <gdaui-decl.h>
+
 static void     widget_embedder_realize       (GtkWidget       *widget);
 static void     widget_embedder_unrealize     (GtkWidget       *widget);
 static void     widget_embedder_get_preferred_width  (GtkWidget *widget,
@@ -208,13 +210,11 @@ widget_embedder_realize (GtkWidget *widget)
 	g_signal_connect (bin->offscreen_window, "from-embedder",
 			  G_CALLBACK (offscreen_window_from_parent), bin);
 
-	GtkStyle *style;
-	style = gtk_widget_get_style (widget);
-	style = gtk_style_attach (style, win);
-	gtk_widget_set_style (widget, style);
+	GtkStyleContext *style;
+	style = gtk_widget_get_style_context (widget);
 
-	gtk_style_set_background (style, win, GTK_STATE_NORMAL);
-	gtk_style_set_background (style, bin->offscreen_window, GTK_STATE_NORMAL);
+	gtk_style_context_set_background (style, win);
+	gtk_style_context_set_background (style, bin->offscreen_window);
 	gdk_window_show (bin->offscreen_window);
 }
 
@@ -353,10 +353,8 @@ widget_embedder_size_allocate (GtkWidget     *widget,
 	}
 
 	if (bin->child && gtk_widget_get_visible (bin->child)){
-		GtkRequisition child_requisition;
 		GtkAllocation child_allocation;
 		
-		gtk_widget_get_child_requisition (bin->child, &child_requisition);
 		child_allocation.x = 0;
 		child_allocation.y = 0;
 		child_allocation.height = h;
@@ -401,7 +399,7 @@ widget_embedder_draw (GtkWidget *widget, cairo_t *cr)
 			cairo_paint (cr);
 
 			if (! bin->valid) {
-				cairo_set_source_rgba (cr, .8, .1, .1, .2);
+				cairo_set_source_rgba (cr, GDAUI_COLOR_UNKNOWN_MASK);
 				cairo_rectangle (cr, child_area.x, child_area.y,
 						 child_area.width, child_area.height);
 				cairo_fill (cr);
