@@ -108,7 +108,7 @@ gdaui_grid_get_type (void)
                         NULL
                 };
 
-		type = g_type_register_static (GTK_TYPE_VBOX, "GdauiGrid", &info, 0);
+		type = g_type_register_static (GTK_TYPE_BOX, "GdauiGrid", &info, 0);
 		g_type_add_interface_static (type, GDAUI_TYPE_DATA_PROXY, &proxy_info);
 		g_type_add_interface_static (type, GDAUI_TYPE_DATA_SELECTOR, &selector_info);
 
@@ -178,6 +178,8 @@ gdaui_grid_init (GdauiGrid *grid)
 	grid->priv->raw_grid = NULL;
 	grid->priv->info = NULL;
 
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (grid), GTK_ORIENTATION_VERTICAL);
+
 	sw = gtk_scrolled_window_new (NULL, NULL);
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
         gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw), GTK_SHADOW_IN);
