@@ -49,6 +49,7 @@ static GError *iter_validate_set_cb (GdaDataModelIter *iter, GdauiRawForm *form)
 static void iter_row_changed_cb (GdaDataModelIter *iter, gint row, GdauiRawForm *form);
 static void proxy_changed_cb (GdaDataProxy *proxy, GdauiRawForm *form);
 static void proxy_reset_cb (GdaDataProxy *proxy, GdauiRawForm *form);
+static void proxy_access_changed_cb (GdaDataProxy *proxy, GdauiRawForm *form);
 static void proxy_row_inserted_or_removed_cb (GdaDataProxy *proxy, gint row, GdauiRawForm *form);
 
 /* GdauiDataProxy interface */
@@ -184,8 +185,7 @@ gdaui_raw_form_class_init (GdauiRawFormClass *class)
 }
 
 static void action_new_cb (GtkAction *action, GdauiRawForm *form);
-static void action_delete_cb (GtkAction *action, GdauiRawForm *form);
-static void action_undelete_cb (GtkAction *action, GdauiRawForm *form);
+static void action_delete_cb (GtkToggleAction *action, GdauiRawForm *form);
 static void action_commit_cb (GtkAction *action, GdauiRawForm *form);
 static void action_reset_cb (GtkAction *action, GdauiRawForm *form);
 static void action_first_record_cb (GtkAction *action, GdauiRawForm *form);
@@ -194,23 +194,19 @@ static void action_next_record_cb (GtkAction *action, GdauiRawForm *form);
 static void action_last_record_cb (GtkAction *action, GdauiRawForm *form);
 static void action_filter_cb (GtkAction *action, GdauiRawForm *form);
 
+static GtkToggleActionEntry ui_actions_t[] = {
+	{ "ActionDelete", GTK_STOCK_REMOVE, "_Delete", NULL, N_("Delete the current record"), G_CALLBACK (action_delete_cb), FALSE},
+};
+
 static GtkActionEntry ui_actions[] = {
-	{ "ActionNew", GTK_STOCK_ADD, "_New", NULL, "Create a new data entry", G_CALLBACK (action_new_cb)},
-	{ "ActionDelete", GTK_STOCK_REMOVE, "_Delete", NULL, "Delete the selected entry", G_CALLBACK (action_delete_cb)},
-	{ "ActionUndelete", GTK_STOCK_UNDELETE, "_Undelete", NULL, "Cancels the deletion of the current entry",
-	  G_CALLBACK (action_undelete_cb)},
-	{ "ActionCommit", GTK_STOCK_SAVE, "_Commit", NULL, "Commit the latest changes", G_CALLBACK (action_commit_cb)},
-	{ "ActionReset", GTK_STOCK_REFRESH, "_Reset", NULL, "Reset the data", G_CALLBACK (action_reset_cb)},
-	{ "ActionFirstRecord", GTK_STOCK_GOTO_FIRST, "_First record", NULL, "Go to first record of records",
-	  G_CALLBACK (action_first_record_cb)},
-	{ "ActionLastRecord", GTK_STOCK_GOTO_LAST, "_Last record", NULL, "Go to last record of records",
-	  G_CALLBACK (action_last_record_cb)},
-	{ "ActionPrevRecord", GTK_STOCK_GO_BACK, "_Previous record", NULL, "Go to previous record of records",
-	  G_CALLBACK (action_prev_record_cb)},
-	{ "ActionNextRecord", GTK_STOCK_GO_FORWARD, "Ne_xt record", NULL, "Go to next record of records",
-	  G_CALLBACK (action_next_record_cb)},
-	{ "ActionFilter", GTK_STOCK_FIND, "Filter", NULL, "Filter records",
-	  G_CALLBACK (action_filter_cb)}
+	{ "ActionNew", GTK_STOCK_ADD, "_New", NULL, N_("Create a new record"), G_CALLBACK (action_new_cb)},
+	{ "ActionCommit", GTK_STOCK_SAVE, "_Commit", NULL, N_("Commit the modifications"), G_CALLBACK (action_commit_cb)},
+	{ "ActionReset", GTK_STOCK_CLEAR, "_Clear", NULL, N_("Clear all the modifications"), G_CALLBACK (action_reset_cb)},
+	{ "ActionFirstRecord", GTK_STOCK_GOTO_FIRST, "_First record", NULL, N_("Go to first record"), G_CALLBACK (action_first_record_cb)},
+	{ "ActionLastRecord", GTK_STOCK_GOTO_LAST, "_Last record", NULL, N_("Go to last record"), G_CALLBACK (action_last_record_cb)},
+	{ "ActionPrevRecord", GTK_STOCK_GO_BACK, "_Previous record", NULL, N_("Go to previous record"), G_CALLBACK (action_prev_record_cb)},
+	{ "ActionNextRecord", GTK_STOCK_GO_FORWARD, "Ne_xt record", NULL, N_("Go to next record"), G_CALLBACK (action_next_record_cb)},
+	{ "ActionFilter", GTK_STOCK_FIND, "Filter", NULL, N_("Filter records"), G_CALLBACK (action_filter_cb)}
 };
 
 static void
@@ -228,7 +224,7 @@ form_activated_cb (GdauiRawForm *form, G_GNUC_UNUSED gpointer data)
 {
 	if (form->priv->write_mode == GDAUI_DATA_PROXY_WRITE_ON_VALUE_ACTIVATED) {
 		gint row;
-
+		
 		row = gda_data_model_iter_get_row (form->priv->iter);
 		if (row >= 0) {
 			/* write back the current row */
@@ -289,6 +285,7 @@ gdaui_raw_form_init (GdauiRawForm *wid)
 	gtk_action_group_set_translation_domain (wid->priv->actions_group, GETTEXT_PACKAGE);
 
         gtk_action_group_add_actions (wid->priv->actions_group, ui_actions, G_N_ELEMENTS (ui_actions), wid);
+        gtk_action_group_add_toggle_actions (wid->priv->actions_group, ui_actions_t, G_N_ELEMENTS (ui_actions_t), wid);
 	action = gtk_action_group_get_action (wid->priv->actions_group, "ActionNew");
 	g_signal_connect (G_OBJECT (action), "activate",
 			  G_CALLBACK (action_new_activated_cb), wid);
@@ -299,7 +296,7 @@ gdaui_raw_form_init (GdauiRawForm *wid)
 
 /**
  * gdaui_raw_form_new:
- * @model: a #GdaDataModel, or %NULL
+ * @model: (allow-none): a #GdaDataModel, or %NULL
  *
  * Creates a new #GdauiRawForm widget to display data in @model
  *
@@ -350,6 +347,8 @@ gdaui_raw_form_dispose (GObject *object)
 							      G_CALLBACK (proxy_changed_cb), form);
 			g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
 							      G_CALLBACK (proxy_reset_cb), form);
+			g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
+							      G_CALLBACK (proxy_access_changed_cb), form);
 			g_object_unref (form->priv->proxy);
 			form->priv->proxy = NULL;
 		}
@@ -421,6 +420,8 @@ gdaui_raw_form_set_property (GObject *object,
 									      G_CALLBACK (proxy_changed_cb), form);
 					g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
 									      G_CALLBACK (proxy_reset_cb), form);
+					g_signal_handlers_disconnect_by_func (G_OBJECT (form->priv->proxy),
+									      G_CALLBACK (proxy_access_changed_cb), form);
 					g_object_unref (G_OBJECT (form->priv->proxy));
 					form->priv->proxy = NULL;
 					form->priv->model = NULL;
@@ -451,6 +452,8 @@ gdaui_raw_form_set_property (GObject *object,
 							  G_CALLBACK (proxy_changed_cb), form);
 					g_signal_connect (G_OBJECT (form->priv->proxy), "reset",
 							  G_CALLBACK (proxy_reset_cb), form);
+					g_signal_connect (G_OBJECT (form->priv->proxy), "access-changed",
+							  G_CALLBACK (proxy_access_changed_cb), form);
 
 					/* we don't want chuncking */
 					g_object_set (object, "paramlist", form->priv->iter, NULL);
@@ -557,6 +560,7 @@ iter_row_changed_cb (GdaDataModelIter *iter, gint row, GdauiRawForm *form)
 							     param, !(attributes & GDA_VALUE_ATTR_NO_MODIF));
 		}
 	}
+	g_signal_emit_by_name (G_OBJECT (form), "selection-changed");
 }
 
 static void
@@ -573,6 +577,13 @@ proxy_reset_cb (GdaDataProxy *proxy, GdauiRawForm *form)
 	g_object_ref (G_OBJECT (proxy));
 	g_object_set (G_OBJECT (form), "model", proxy, NULL);
 	g_object_unref (G_OBJECT (proxy));
+	g_signal_emit_by_name (G_OBJECT (form), "selection-changed");
+}
+
+static void
+proxy_access_changed_cb (G_GNUC_UNUSED GdaDataProxy *proxy, GdauiRawForm *form)
+{
+	iter_row_changed_cb (form->priv->iter, gda_data_model_iter_get_row (form->priv->iter), form);
 }
 
 static void
@@ -639,38 +650,36 @@ action_new_cb (G_GNUC_UNUSED GtkAction *action, GdauiRawForm *form)
 }
 
 static void
-action_delete_cb (G_GNUC_UNUSED GtkAction *action, GdauiRawForm *form)
+action_delete_cb (GtkToggleAction *action, GdauiRawForm *form)
 {
-	gint row;
-
-	row = gda_data_model_iter_get_row (form->priv->iter);
-	g_return_if_fail (row >= 0);
-	gda_data_proxy_delete (form->priv->proxy, row);
-
-	if (form->priv->write_mode >= GDAUI_DATA_PROXY_WRITE_ON_ROW_CHANGE) {
-		/* force the proxy to apply the current row deletion */
-		gint newrow;
-
-		newrow = gda_data_model_iter_get_row (form->priv->iter);
-		if (row == newrow) {/* => row has been marked as delete but not yet really deleted */
-			GError *error = NULL;
-			if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
-				_gdaui_utility_display_error ((GdauiDataProxy *) form, TRUE, error);
-				if (error)
-					g_error_free (error);
+	if (gtk_toggle_action_get_active (action)) {
+		gint row;
+		row = gda_data_model_iter_get_row (form->priv->iter);
+		g_return_if_fail (row >= 0);
+		gda_data_proxy_delete (form->priv->proxy, row);
+		
+		if (form->priv->write_mode >= GDAUI_DATA_PROXY_WRITE_ON_ROW_CHANGE) {
+			/* force the proxy to apply the current row deletion */
+			gint newrow;
+			
+			newrow = gda_data_model_iter_get_row (form->priv->iter);
+			if (row == newrow) {/* => row has been marked as delete but not yet really deleted */
+				GError *error = NULL;
+				if (!gda_data_proxy_apply_row_changes (form->priv->proxy, row, &error)) {
+					_gdaui_utility_display_error ((GdauiDataProxy *) form, TRUE, error);
+					if (error)
+						g_error_free (error);
+				}
 			}
 		}
 	}
-}
-
-static void
-action_undelete_cb (G_GNUC_UNUSED GtkAction *action, GdauiRawForm *form)
-{
-	gint row;
-
-	row = gda_data_model_iter_get_row (form->priv->iter);
-	g_return_if_fail (row >= 0);
-	gda_data_proxy_undelete (form->priv->proxy, row);
+	else {
+		gint row;
+		
+		row = gda_data_model_iter_get_row (form->priv->iter);
+		g_return_if_fail (row >= 0);
+		gda_data_proxy_undelete (form->priv->proxy, row);
+	}
 }
 
 static void
@@ -710,7 +719,7 @@ action_commit_cb (G_GNUC_UNUSED GtkAction *action, GdauiRawForm *form)
 	}
 
 	/* get to a correct selected row */
-	for (; !gda_data_model_iter_move_to_row (form->priv->iter, row) && (row >= 0); row--);
+	for (; (row >= 0) &&!gda_data_model_iter_move_to_row (form->priv->iter, row); row--);
 }
 
 static void
