@@ -87,6 +87,7 @@ struct _GdauiRtEditorPriv
 
 /* get a pointer to the parents to be able to call their destructor */
 static GObjectClass *parent_class = NULL;
+static GtkCssProvider *css_provider = NULL;
 static gchar *help_str=N_("\"\"\"= Title level 1 =\n"
 			  "== Title level 2 ==\n"
 			  "\"\"\"= Title level 1 =\n"
@@ -388,6 +389,20 @@ gdaui_rt_editor_init (GdauiRtEditor *rte)
 
 	/* toolbar */
 	toolbar = gtk_ui_manager_get_widget (rte->priv->uimanager, "/ToolBar");
+
+	if (!css_provider) {
+		css_provider = gtk_css_provider_new ();
+		gtk_css_provider_load_from_data (css_provider,
+						 "* {\n"
+						 "-GtkToolbar-shadow-type : GTK_SHADOW_NONE;\n"
+						 "xthickness : 0;\n"
+						 "ythickness : 0}",
+						 -1, NULL);
+	}
+	gtk_style_context_add_provider (gtk_widget_get_style_context ((GtkWidget*) toolbar),
+					GTK_STYLE_PROVIDER (css_provider),
+					GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
 	gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_MENU);
 	rte->priv->toolbar = toolbar;
 	gtk_box_pack_end (GTK_BOX (rte), toolbar, FALSE, FALSE, 0);
@@ -1247,7 +1262,7 @@ spaces_since_start_of_line (GtkTextIter *iter)
  * get_markup_token
  * @iter: starting position
  * @out_nb_spaces_before: a place to set the value returned by spaces_since_start_of_line() if called
- * @out_end: place to put the last used position, or %NULL
+ * @out_end: (allow-none): place to put the last used position, or %NULL
  * @rte: the #GdauiRtEditor
  *
  * Parses marking tokens, nothing else
@@ -1904,6 +1919,7 @@ gdaui_rt_editor_set_editable (GdauiRtEditor *editor, gboolean editable)
 {
 	g_return_if_fail (GDAUI_IS_RT_EDITOR (editor));
 	gtk_text_view_set_editable (editor->priv->textview, editable);
+	gtk_text_view_set_cursor_visible (editor->priv->textview, editable);
 	show_hide_toolbar (editor);
 }
 
