@@ -1,21 +1,21 @@
-/* gdaui-server-operation.c
+/*
+ * Copyright (C) 2009 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * Copyright (C) 2006 - 2010 Vivien Malerba
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This program is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU Library General Public License for more details.
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
- * USA
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <stdlib.h>
@@ -174,7 +174,7 @@ gdaui_server_operation_get_type (void)
 			0
 		};
 
-		type = g_type_register_static (GTK_TYPE_VBOX, "GdauiServerOperation", &info, 0);
+		type = g_type_register_static (GTK_TYPE_BOX, "GdauiServerOperation", &info, 0);
 	}
 
 	return type;
@@ -215,6 +215,8 @@ gdaui_server_operation_init (GdauiServerOperation * wid)
 	wid->priv->glade = NULL;
 #endif
 	wid->priv->opt_header = FALSE;
+
+	gtk_orientable_set_orientation (GTK_ORIENTABLE (wid), GTK_ORIENTATION_VERTICAL);
 }
 
 
@@ -476,7 +478,7 @@ fill_create_widget (GdauiServerOperation *form, const gchar *path, gchar **secti
 		winfo = gdaui_data_proxy_info_new (GDAUI_DATA_PROXY (grid),
 						   GDAUI_DATA_PROXY_INFO_ROW_MODIFY_BUTTONS);
 
-		box = gtk_vbox_new (FALSE, 0);
+		box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 		gtk_box_pack_start (GTK_BOX (box), plwid, TRUE, TRUE, 0);
 		gtk_widget_show (plwid);
 
@@ -767,7 +769,7 @@ gdaui_server_operation_fill (GdauiServerOperation *form)
 				gtk_label_set_markup (GTK_LABEL (label), section_str);
 				g_free (section_str);
 
-				hbox = gtk_hbox_new (FALSE, 0); /* HIG */
+				hbox = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 0); /* HIG */
 				gtk_widget_show (hbox);
 				lab = gtk_label_new ("    ");
 				gtk_box_pack_start (GTK_BOX (hbox), lab, FALSE, FALSE, 0);
@@ -940,7 +942,7 @@ sequence_item_added_cb (GdaServerOperation *op, const gchar *seq_path, gint item
 			}
 
 			/* move children DOWN if necessary and change the "_index" property */
-			if (top_attach >= item_index) {
+			if (top_attach >= (guint)item_index) {
 				struct MoveChild *mc;
 				gint index;
 
@@ -1016,7 +1018,7 @@ sequence_item_remove_cb (GdaServerOperation *op, const gchar *seq_path, gint ite
 			guint top_attach;
 			gtk_container_child_get (GTK_CONTAINER (table), child,
 						 "top-attach", &top_attach, NULL);
-			if (top_attach == item_index) {
+			if (top_attach == (guint)item_index) {
 				gtk_widget_destroy (child);
 				g_list_free (children);
 				children = gtk_container_get_children (GTK_CONTAINER (table));
@@ -1046,7 +1048,7 @@ sequence_item_remove_cb (GdaServerOperation *op, const gchar *seq_path, gint ite
 			}
 
 			/* move widgets UP if necessary and change the "_index" property */
-			if (top_attach > item_index) {
+			if (top_attach > (guint)item_index) {
 				struct MoveChild *mc;
 				gint index;
 
@@ -1083,9 +1085,9 @@ sequence_item_remove_cb (GdaServerOperation *op, const gchar *seq_path, gint ite
 /**
  * gdaui_server_operation_new_in_dialog
  * @op: a #GdaServerOperation object
- * @parent: the parent window for the new dialog, or %NULL
- * @title: the title of the dialog window, or %NULL
- * @header: a helper text displayed at the top of the dialog, or %NULL
+ * @parent: (allow-none): the parent window for the new dialog, or %NULL
+ * @title: (allow-none): the title of the dialog window, or %NULL
+ * @header: (allow-none): a helper text displayed at the top of the dialog, or %NULL
  *
  * Creates a new #GdauiServerOperation widget in the same way as gdaui_server_operation_new()
  * and puts it into a #GtkDialog widget. The returned dialog has the "Ok" and "Cancel" buttons
@@ -1162,10 +1164,10 @@ create_table_fields_array_create_widget (GdauiServerOperation *form, const gchar
 	info_node = gda_server_operation_get_node_info (form->priv->op, path);
 	g_assert (info_node->type == GDA_SERVER_OPERATION_NODE_DATA_MODEL);
 
-	hlayout = gtk_hpaned_new ();
+	hlayout = gtk_paned_new (GTK_ORIENTATION_HORIZONTAL);
 
 	/* form for field properties */
-	box = gtk_vbox_new (FALSE, 0);
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_paned_pack2 (GTK_PANED (hlayout), box, TRUE, TRUE);
 
 	label = gtk_label_new (_("<b>Field properties:</b>"));
@@ -1184,7 +1186,7 @@ create_table_fields_array_create_widget (GdauiServerOperation *form, const gchar
 	gtk_widget_show_all (box);
 
 	/* grid for field names */
-	box = gtk_vbox_new (FALSE, 0);
+	box = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	gtk_paned_pack1 (GTK_PANED (hlayout), box, TRUE, TRUE);
 
 	label = gtk_label_new (_("<b>Fields:</b>"));
