@@ -1,23 +1,22 @@
-/* 
- * Copyright (C) 2007 - 2011 The GNOME Foundation.
+/*
+ * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2009 Murray Cumming <murrayc@murrayc.com>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -34,7 +33,7 @@
 gchar *
 gda_sql_value_stringify (const GValue *value)
 {
-        if (value && G_IS_VALUE (value)) {
+        if (value && !GDA_VALUE_HOLDS_NULL (value)) {
                 if (g_value_type_transformable (G_VALUE_TYPE (value), G_TYPE_STRING)) {
                         GValue *string;
                         gchar *str;
@@ -42,8 +41,7 @@ gda_sql_value_stringify (const GValue *value)
                         string = g_value_init (g_new0 (GValue, 1), G_TYPE_STRING);
                         g_value_transform (value, string);
                         str = g_value_dup_string (string);
-                        g_value_unset (string);
-			g_free (string);
+			gda_value_free (string);
                         return str;
                 }
                 else {
