@@ -1,24 +1,21 @@
 /*
- * Copyright (C) 2008 - 2011 The GNOME Foundation
+ * Copyright (C) 2008 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *         Vivien Malerba <malerba@gnome-db.org>
- *         Bas Driessen <bas.driessen@xobas.com>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <string.h>
@@ -984,7 +981,7 @@ fill_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata,
 		const gchar *this_col_name;
 		const GValue *this_col_pname;
 		GValue *nthis_col_pname;
-		GType gtype = 0;
+		GType gtype = G_TYPE_INVALID;
 		const GValue *cvalue;
 		
 		this_col_pname = gda_data_model_get_value_at (tmpmodel, 1, i, error);
@@ -1061,7 +1058,7 @@ fill_columns_model (GdaConnection *cnc, SqliteConnectionData *cdata,
 			gtype = pg ? (GType) *pg : GDA_TYPE_NULL;
 			g_free (tmp);
 		}
-		if (gtype == 0) 
+		if (gtype == G_TYPE_INVALID)
 			/* default to string if nothing else */
 			g_value_set_string ((v6 = gda_value_new (G_TYPE_STRING)), "string");
 		else
