@@ -1,24 +1,22 @@
-/* 
- * GDA common library
- * Copyright (C) 2007 - 2011 The GNOME Foundation.
+/*
+ * Copyright (C) 2007 - 2011 Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2009 Bas Driessen <bas.driessen@xobas.com>
+ * Copyright (C) 2010 David King <davidk@openismus.com>
  *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * This library is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU Lesser General Public
+ * License as published by the Free Software Foundation; either
+ * version 2 of the License, or (at your option) any later version.
  *
- * This Library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public License as
- * published by the Free Software Foundation; either version 2 of the
- * License, or (at your option) any later version.
- *
- * This Library is distributed in the hope that it will be useful,
+ * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this Library; see the file COPYING.LIB.  If not,
- * write to the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
- * Boston, MA 02111-1307, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <glib/gi18n-lib.h>
@@ -28,6 +26,7 @@
 #include "gda-vconnection-data-model-private.h"
 #include <sqlite3.h>
 #include <libgda/gda-connection-private.h>
+#include <libgda/gda-connection-internal.h>
 #include <libgda/gda-data-model-iter.h>
 #include <libgda/gda-blob-op.h>
 #include "../gda-sqlite.h"
@@ -52,6 +51,12 @@ static gboolean       gda_vprovider_data_model_open_connection (GdaServerProvide
 								gpointer cb_data);
 static gboolean       gda_vprovider_data_model_close_connection (GdaServerProvider *provider,
 								 GdaConnection *cnc);
+static GObject        *gda_vprovider_data_model_statement_execute (GdaServerProvider *provider, GdaConnection *cnc,
+								   GdaStatement *stmt, GdaSet *params,
+								   GdaStatementModelUsage model_usage,
+								   GType *col_types, GdaSet **last_inserted_row,
+								   guint *task_id, GdaServerProviderExecCallback async_cb,
+								   gpointer cb_data, GError **error);
 static const gchar   *gda_vprovider_data_model_get_name (GdaServerProvider *provider);
 
 /*
@@ -69,6 +74,7 @@ gda_vprovider_data_model_class_init (GdaVproviderDataModelClass *klass)
 	server_class->create_connection = gda_vprovider_data_model_create_connection;
 	server_class->open_connection = gda_vprovider_data_model_open_connection;
 	server_class->close_connection = gda_vprovider_data_model_close_connection;
+	server_class->statement_execute = gda_vprovider_data_model_statement_execute;
 
 	server_class->get_name = gda_vprovider_data_model_get_name;
 
@@ -191,6 +197,7 @@ typedef struct {
 
 typedef struct {
 	sqlite3_vtab_cursor      base; /* base.pVtab is a pointer to the sqlite3_vtab virtual table */
+	GdaDataModel            *model; /* in which @iter iterates */
 	GdaDataModelIter        *iter;
 } VirtualCursor;
 
@@ -237,6 +244,48 @@ static sqlite3_module Module = {
 	virtualRename                 /* Rename - Notification that the table will be given a new name */
 };
 
+/*
+ * handle data model exceptions and return appropriate code
+ */
+static int
+handle_data_model_exception (sqlite3_vtab *pVtab, GdaDataModel *model)
+{
+	GError **exceptions;
+	gint i;
+	exceptions = gda_data_model_get_exceptions (model);
+	if (!exceptions)
+		return SQLITE_OK;
+
+	GError *trunc_error = NULL;
+	GError *fatal_error = NULL;
+	for (i = 0; exceptions [i]; i++) {
+		GError *e;
+		e = exceptions [i];
+		if ((e->domain == GDA_DATA_MODEL_ERROR) &&
+		    (e->code == GDA_DATA_MODEL_TRUNCATED_ERROR))
+			trunc_error = e;
+		else {
+			fatal_error = e;
+			break;
+		}
+	}
+	if (fatal_error || trunc_error) {
+		GError *e;
+		e = fatal_error;
+		if (!e)
+			e = trunc_error;
+		if (pVtab->zErrMsg)
+			SQLITE3_CALL (sqlite3_free) (pVtab->zErrMsg);
+		pVtab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
+			(e->message ? e->message : _("No detail"));
+		if (fatal_error)
+			return SQLITE_ERROR;
+		else
+			return SQLITE_IOERR_TRUNCATE;
+	}
+	return SQLITE_OK;
+}
+
 static GdaConnection *
 gda_vprovider_data_model_create_connection (GdaServerProvider *provider)
 {
@@ -265,10 +314,10 @@ gda_vprovider_data_model_open_connection (GdaServerProvider *provider, GdaConnec
 
 	if (params) {
 		m_params = gda_quark_list_copy (params);
-		gda_quark_list_add_from_string (m_params, "_IS_VIRTUAL=TRUE;LOAD_GDA_FUNCTIONS=TRUE", TRUE);
+		gda_quark_list_add_from_string (m_params, "_IS_VIRTUAL=TRUE;EXTRA_FUNCTIONS=TRUE", TRUE);
 	}
 	else
-		m_params = gda_quark_list_new_from_string ("_IS_VIRTUAL=TRUE;LOAD_GDA_FUNCTIONS=TRUE");
+		m_params = gda_quark_list_new_from_string ("_IS_VIRTUAL=TRUE;EXTRA_FUNCTIONS=TRUE");
 
 	if (! GDA_SERVER_PROVIDER_CLASS (parent_class)->open_connection (GDA_SERVER_PROVIDER (provider), cnc, m_params,
 									 auth, NULL, NULL, NULL)) {
@@ -313,6 +362,73 @@ gda_vprovider_data_model_close_connection (GdaServerProvider *provider, GdaConne
 	return GDA_SERVER_PROVIDER_CLASS (parent_class)->close_connection (GDA_SERVER_PROVIDER (provider), cnc);
 }
 
+static GObject *
+gda_vprovider_data_model_statement_execute (GdaServerProvider *provider, GdaConnection *cnc,
+					    GdaStatement *stmt, GdaSet *params,
+					    GdaStatementModelUsage model_usage,
+					    GType *col_types, GdaSet **last_inserted_row,
+					    guint *task_id, GdaServerProviderExecCallback async_cb,
+					    gpointer cb_data, GError **error)
+{
+	GObject *retval;
+	if (async_cb) {
+                g_set_error (error, GDA_SERVER_PROVIDER_ERROR, GDA_SERVER_PROVIDER_METHOD_NON_IMPLEMENTED_ERROR,
+                             "%s", _("Provider does not support asynchronous statement execution"));
+                return NULL;
+        }
+        
+	retval = GDA_SERVER_PROVIDER_CLASS (parent_class)->statement_execute (provider, cnc, stmt, params,
+									      model_usage, col_types,
+									      last_inserted_row, task_id,
+									      async_cb, cb_data, error);
+	if (retval) {
+		gchar *sql;
+		sql = gda_statement_to_sql (stmt, params, NULL);
+		if (sql) {
+			gchar *ptr = NULL;
+			/* look for DROP TABLE to signal table removal */
+			if (! g_ascii_strncasecmp (sql, "DROP", 4))
+				ptr = sql + 4;
+			else if (! g_ascii_strncasecmp (sql, "CREATE", 6))
+				ptr = sql + 6;
+			if (ptr) {
+				/* skip spaces */
+				for (; *ptr && (g_ascii_isspace (*ptr) || (*ptr == '\n')); ptr++);
+
+				if (! g_ascii_strncasecmp (ptr, "TABLE", 5)) {
+					/* skip spaces */
+					gchar delim = 0;
+					gchar *table_name, *quoted;
+					for (ptr = ptr+5;
+					     *ptr && (g_ascii_isspace (*ptr) || (*ptr == '\n'));
+					     ptr++);
+					if (*ptr == '\'') {
+						delim = '\'';
+						ptr++;
+					}
+					else if (*ptr == '"') {
+						delim = '"';
+						ptr++;
+					}
+					table_name = ptr;
+					if (delim)
+						for (; *ptr && (*ptr != delim) ; ptr++);
+					else
+						for (; *ptr && g_ascii_isalnum (*ptr); ptr++);
+					*ptr = 0;
+					quoted = _gda_connection_compute_table_virtual_name (GDA_CONNECTION (cnc), table_name);
+					/*g_print ("%s() emits the 'vtable-dropped' signal for table [%s]\n",
+					  __FUNCTION__, quoted);*/
+					g_signal_emit_by_name (cnc, "vtable-dropped", quoted);
+					g_free (quoted);
+				}
+			}
+			g_free (sql);
+		}
+	}
+	return retval;
+}
+
 static const gchar *
 gda_vprovider_data_model_get_name (G_GNUC_UNUSED GdaServerProvider *provider)
 {
@@ -341,7 +457,11 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 
 	td = gda_vconnection_get_table_data_by_unique_name (cnc, spec_name);
 	g_free (spec_name);
-	g_assert (td);
+	if (!td) {
+		/* wrong usage! */
+		*pzErr = SQLITE3_CALL (sqlite3_mprintf) (_("Wrong usage of Libgda's virtual tables"));
+		return SQLITE_ERROR;
+	}
 
 	/* preparations */
 	if (td->spec->data_model) {
@@ -357,7 +477,7 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 	GError *error = NULL;
 	if (!td->columns && td->spec->create_columns_func)
 		td->columns = td->spec->create_columns_func (td->spec, &error);
-	if (! td->columns) {
+	if (!td->columns) {
 		if (error && error->message) {
 			int len = strlen (error->message) + 1;
 			*pzErr = SQLITE3_CALL (sqlite3_malloc) (sizeof (gchar) * len);
@@ -384,10 +504,7 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 
 		if (i != 0)
 			g_string_append (sql, ", ");
-		if (td->columns)
-			column = g_list_nth_data (td->columns, i);
-		else
-			column = gda_data_model_describe_column (td->spec->data_model, i);
+		column = g_list_nth_data (td->columns, i);
 		if (!column) {
 			*pzErr = SQLITE3_CALL (sqlite3_mprintf) (_("Can't get data model description for column %d"), i);
 			g_string_free (sql, TRUE);
@@ -403,12 +520,14 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 		gtype = gda_column_get_g_type (column);
 		type = g_type_name (gtype);
 		if (!type) {
-			*pzErr = SQLITE3_CALL (sqlite3_mprintf) (_("Can't get data model's column type or type for column %d"), i);
+			*pzErr = SQLITE3_CALL (sqlite3_mprintf) (_("Can't get data model's column type for column %d"), i);
 			g_string_free (sql, TRUE);
 			return SQLITE_ERROR;
 		}
-		else if ((gtype == GDA_TYPE_BLOB) || (gtype == GDA_TYPE_BINARY))
+		else if (gtype == GDA_TYPE_BLOB)
 			type = "blob";
+		else if (gtype == GDA_TYPE_BINARY)
+			type = "binary";
 		else if (gtype == G_TYPE_STRING)
 			type = "string";
 		else if ((gtype == G_TYPE_INT) || (gtype == G_TYPE_UINT) || 
@@ -433,6 +552,8 @@ virtualCreate (sqlite3 *db, void *pAux, int argc, const char *const *argv, sqlit
 		g_string_append (sql, type);
 		if (! gda_column_get_allow_null (column)) 
 			g_string_append (sql, " NOT NULL");
+		if (gtype == G_TYPE_STRING)
+			g_string_append (sql, " COLLATE LOCALE");
 	}
 
 	/* add a hidden column which contains the row number of the data model */
@@ -493,15 +614,14 @@ virtualDestroy (sqlite3_vtab *pVtab)
 }
 
 static int
-virtualOpen (sqlite3_vtab *pVTab, sqlite3_vtab_cursor **ppCursor)
+virtualOpen (G_GNUC_UNUSED sqlite3_vtab *pVTab, sqlite3_vtab_cursor **ppCursor)
 {
 	VirtualCursor *cursor;
 
-	TRACE (pVTab, NULL);
-
 	/* create empty cursor */
 	cursor = g_new0 (VirtualCursor, 1);
 	*ppCursor = (sqlite3_vtab_cursor*) cursor;
+	TRACE (pVTab, cursor);
 
 	return SQLITE_OK;
 }
@@ -515,8 +635,11 @@ virtualClose (sqlite3_vtab_cursor *cur)
 
 	if (cursor->iter)
 		g_object_unref (cursor->iter);
+	if (cursor->model)
+		g_object_unref (cursor->model);
 
 	g_free (cur);
+
 	return SQLITE_OK;
 }
 
@@ -541,13 +664,14 @@ virtualNext (sqlite3_vtab_cursor *cur)
 
 	TRACE (cur->pVtab, cur);
 
+	if (gda_data_model_get_exceptions (cursor->model))
+		return SQLITE_IOERR_TRUNCATE;
+
 	if (!gda_data_model_iter_move_next (cursor->iter)) {
 		if (gda_data_model_iter_is_valid (cursor->iter))
 			return SQLITE_IOERR;
-		else
-			return SQLITE_OK;
 	}
-	return SQLITE_OK;
+	return handle_data_model_exception (cur->pVtab, cursor->model);
 }
 
 static int
@@ -617,7 +741,7 @@ virtualRowid (sqlite3_vtab_cursor *cur, sqlite_int64 *pRowid)
 	*pRowid = gda_data_model_iter_get_row (cursor->iter);
 	if (! vtable->rowid_hash || (vtable->rowid_hash_model == vtable->td->real_model)) {
 		if (! vtable->rowid_hash) {
-			vtable->rowid_hash = g_hash_table_new_full (g_int_hash, g_int_equal,
+			vtable->rowid_hash = g_hash_table_new_full (g_int64_hash, g_int64_equal,
 								    g_free,
 								    (GDestroyNotify) g_object_unref);
 			vtable->rowid_hash_model = vtable->td->real_model;
@@ -634,10 +758,12 @@ virtualRowid (sqlite3_vtab_cursor *cur, sqlite_int64 *pRowid)
 			GValue *gvalue;
 			cvalue = gda_data_model_iter_get_value_at (cursor->iter, i);
 			gvalue = gda_row_get_value (grow, i);
-			if (G_VALUE_TYPE (cvalue) != GDA_TYPE_NULL) {
-				g_value_init (gvalue, G_VALUE_TYPE (cvalue));
+			if (cvalue) {
+				gda_value_reset_with_type (gvalue, G_VALUE_TYPE (cvalue));
 				g_value_copy (cvalue, gvalue);
 			}
+			else
+				gda_row_invalidate_value (grow, gvalue);
 		}
 		
 		hid = g_new (gint64, 1);
@@ -680,6 +806,7 @@ create_value_from_sqlite3_value_notype (sqlite3_value *svalue)
 		break;
 	}
 	case SQLITE_NULL:
+		gda_value_set_null (value);
 		break;
 	case SQLITE3_TEXT:
 	default:
@@ -779,8 +906,10 @@ virtualFilter (sqlite3_vtab_cursor *pVtabCursor, int idxNum, const char *idxStr,
 	else
 		cursor->iter = gda_data_model_create_iter (vtable->td->real_model);
 
+	cursor->model = g_object_ref (vtable->td->real_model);
 	gda_data_model_iter_move_next (cursor->iter);
-	return SQLITE_OK;
+
+	return handle_data_model_exception (pVtabCursor->pVtab, cursor->model);
 }
 
 #ifdef GDA_DEBUG_VIRTUAL
@@ -827,43 +956,48 @@ index_info_dump (sqlite3_index_info *pIdxInfo, gboolean dump_out)
 static void
 map_sqlite3_info_to_gda_filter (sqlite3_index_info *info, GdaVconnectionDataModelFilter *filter)
 {
-	gint i;
 	memset (filter, 0, sizeof (GdaVconnectionDataModelFilter));
-	filter->nConstraint = info->nConstraint;
 	if (info->nConstraint > 0) {
-		filter->aConstraint = g_new (struct GdaVirtualConstraint, filter->nConstraint);
-		filter->aConstraintUsage = g_new (struct GdaVirtualConstraintUsage, filter->nConstraint);
-		for (i = 0; i < info->nConstraint; i++) {
-			filter->aConstraint[i].iColumn = info->aConstraint[i].iColumn;
+		gint i, j;
+		filter->aConstraint = g_new (struct GdaVirtualConstraint, info->nConstraint);
+		filter->aConstraintUsage = g_new (struct GdaVirtualConstraintUsage, info->nConstraint);
+		for (i = 0, j = 0; i < info->nConstraint; i++) {
+			if (! info->aConstraint[i].usable)
+				continue;
+
+			filter->aConstraint[j].iColumn = info->aConstraint[i].iColumn;
 			switch (info->aConstraint[i].op) {
 			case SQLITE_INDEX_CONSTRAINT_EQ:
-				filter->aConstraint[i].op = GDA_SQL_OPERATOR_TYPE_EQ;
+				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_EQ;
 				break;
 			case SQLITE_INDEX_CONSTRAINT_GT:
-				filter->aConstraint[i].op = GDA_SQL_OPERATOR_TYPE_GT;
+				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_GT;
 				break;
 			case SQLITE_INDEX_CONSTRAINT_LE:
-				filter->aConstraint[i].op = GDA_SQL_OPERATOR_TYPE_LEQ;
+				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_LEQ;
 				break;
 			case SQLITE_INDEX_CONSTRAINT_LT:
-				filter->aConstraint[i].op = GDA_SQL_OPERATOR_TYPE_LT;
+				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_LT;
 				break;
 			case SQLITE_INDEX_CONSTRAINT_GE:
-				filter->aConstraint[i].op = GDA_SQL_OPERATOR_TYPE_GEQ;
+				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_GEQ;
 				break;
 			case SQLITE_INDEX_CONSTRAINT_MATCH:
-				filter->aConstraint[i].op = GDA_SQL_OPERATOR_TYPE_REGEXP;
+				filter->aConstraint[j].op = GDA_SQL_OPERATOR_TYPE_REGEXP;
 				break;
 			default:
 				g_assert_not_reached ();
 			}
 
-			filter->aConstraintUsage[i].argvIndex = 0;
-			filter->aConstraintUsage[i].omit = FALSE;
+			filter->aConstraintUsage[j].argvIndex = 0;
+			filter->aConstraintUsage[j].omit = FALSE;
+			j++;
 		}
+		filter->nConstraint = j;
 	}
 	filter->nOrderBy = info->nOrderBy;
 	if (filter->nOrderBy > 0) {
+		gint i;
 		filter->aOrderBy = g_new (struct GdaVirtualOrderby, filter->nOrderBy);
 		for (i = 0; i < info->nOrderBy; i++) {
 			filter->aOrderBy[i].iColumn = info->aOrderBy[i].iColumn;
@@ -882,12 +1016,15 @@ map_sqlite3_info_to_gda_filter (sqlite3_index_info *info, GdaVconnectionDataMode
 static void
 map_consume_gda_filter_to_sqlite3_info (GdaVconnectionDataModelFilter *filter, sqlite3_index_info *info)
 {
-	gint i;
 	g_assert (filter->nConstraint == info->nConstraint);
 	if (info->nConstraint > 0) {
-		for (i = 0; i < info->nConstraint; i++) {
-			info->aConstraintUsage[i].argvIndex = filter->aConstraintUsage[i].argvIndex;
-			info->aConstraintUsage[i].omit = filter->aConstraintUsage[i].omit ? 1 : 0;
+		gint i, j;
+		for (i = 0, j = 0; i < info->nConstraint; i++) {
+			if (! info->aConstraint[i].usable)
+				continue;
+			info->aConstraintUsage[i].argvIndex = filter->aConstraintUsage[j].argvIndex;
+			info->aConstraintUsage[i].omit = filter->aConstraintUsage[j].omit ? 1 : 0;
+			j++;
 		}
 		g_free (filter->aConstraint);
 		g_free (filter->aConstraintUsage);
@@ -922,7 +1059,6 @@ virtualBestIndex (sqlite3_vtab *tab, sqlite3_index_info *pIdxInfo)
 #endif
 	}
 
-
 	return SQLITE_OK;
 }
 
@@ -1033,16 +1169,14 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 		if (! vtable->td->modif_stmt [ptype]) {
 			if (! stmt) {
 				tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-					(_("No statement provided to modify the data model "
-					   "representing the table"));
+					(_("No statement provided to modify the data"));
 				return SQLITE_READONLY;
 			}
 		
 			GdaSet *params;
 			if (! gda_statement_get_parameters (stmt, &params, NULL) || !params) {
 				tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-					(_("Invalid statement provided to modify the data model "
-					   "representing the table"));
+					(_("Invalid statement provided to modify the data"));
 				g_object_unref (stmt);
 				return SQLITE_READONLY;
 			}
@@ -1061,8 +1195,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 			id = gda_holder_get_id (holder);
 			if (!id) {
 				tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-					(_("Invalid parameter in statement provided to modify "
-					   "the data model representing the table"));
+					(_("Invalid parameter in statement to modify the data"));
 				return SQLITE_READONLY;
 			}
 			if (*id == '+' && id[1]) {
@@ -1115,8 +1248,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 				if (! exec_set) {
 					/* can't give value to param named @id */
 					tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-						(_("Invalid parameter in statement provided to modify "
-						   "the data model representing the table"));
+						(_("Invalid parameter in statement to modify the data"));
 					return SQLITE_READONLY;
 				}
 				eh = gda_set_get_holder (exec_set, id);
@@ -1124,8 +1256,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 				    ! gda_holder_set_bind (holder, eh, NULL)) {
 					/* can't give value to param named @id */
 					tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-						(_("Invalid parameter in statement provided to modify "
-						   "the data model representing the table"));
+						(_("Invalid parameter in statement to modify the data"));
 					return SQLITE_READONLY;
 				}
 			}
@@ -1133,14 +1264,14 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 
 		GdaConnection *cnc;
 		cnc = gda_data_select_get_connection (GDA_DATA_SELECT (vtable->td->real_model));
-		
+
+		GError *lerror = NULL;
 #ifdef GDA_DEBUG_NO
 		gchar *sql;
-		GError *lerror = NULL;
 		sql = gda_statement_to_sql (stmt, NULL, NULL);
 		g_print ("SQL: [%s] ", sql);
 		g_free (sql);
-		sql = gda_statement_to_sql_extended (stmt, cnc, params, GDA_STATEMENT_SQL_PRETTY, NULL, &lerror);
+		sql = gda_statement_to_sql_extended (stmt, cnc, vtable->td->modif_params [ptype], GDA_STATEMENT_SQL_PRETTY, NULL, &lerror);
 		if (sql) {
 			g_print ("With params: [%s]\n", sql);
 			g_free (sql);
@@ -1154,11 +1285,12 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 		if (!cnc ||
 		    (gda_connection_statement_execute_non_select (cnc, stmt,
 								  vtable->td->modif_params [ptype],
-								  NULL, NULL) == -1)) {
+								  NULL, &lerror) == -1)) {
 			/* failed to execute */
 			tab->zErrMsg = SQLITE3_CALL (sqlite3_mprintf)
-				(_("Failed to execute the statement provided to modify "
-				   "the data model representing the table"));
+				(_("Failed to modify data: %s"),
+				 lerror && lerror->message ? lerror->message : _("No detail"));
+			g_clear_error (&lerror);
 			return SQLITE_READONLY;
 		}
 		return SQLITE_OK;
@@ -1248,7 +1380,7 @@ virtualUpdate (sqlite3_vtab *tab, int nData, sqlite3_value **apData, sqlite_int6
 }
 
 static int
-virtualBegin (sqlite3_vtab *tab)
+virtualBegin (G_GNUC_UNUSED sqlite3_vtab *tab)
 {
 	TRACE (tab, NULL);
 	/* no documentation currently available, don't do anything */
@@ -1287,7 +1419,7 @@ virtualRollback (G_GNUC_UNUSED sqlite3_vtab *tab)
 }
 
 static int
-virtualRename (sqlite3_vtab *pVtab, G_GNUC_UNUSED const char *zNew)
+virtualRename (G_GNUC_UNUSED sqlite3_vtab *pVtab, G_GNUC_UNUSED const char *zNew)
 {
 	TRACE (pVtab, NULL);
 	/* not yet analysed and implemented */
