@@ -1,22 +1,20 @@
 /*
- * Copyright (C) 2011 The GNOME Foundation
- *
- * AUTHORS:
- *      Vivien Malerba <malerba@gnome-db.org>
+ * Copyright (C) 2011 Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
- * modify it under the terms of the GNU Library General Public
+ * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
- * Library General Public License for more details.
+ * Lesser General Public License for more details.
  *
- * You should have received a copy of the GNU Library General Public
- * License along with this library; if not, write to the Free
- * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * You should have received a copy of the GNU Lesser General Public
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+ * Boston, MA  02110-1301, USA.
  */
 
 #include <glib-object.h>
@@ -525,8 +523,8 @@ static gint classes_sort (GdaLdapClass *lcl1, GdaLdapClass *lcl2);
 
 /**
  * gdaprov_ldap_get_class_info:
- * @cdata:
- * @class:
+ * @cnc: a #GdaLdapConnection (not %NULL)
+ * @classname: the class name (not %NULL)
  *
  * Returns: the #GdaLdapClass for @classname, or %NULL
  */
@@ -721,6 +719,22 @@ gdaprov_ldap_get_class_info (GdaLdapConnection *cnc, const gchar *classname)
 	return retval;
 }
 
+static gint
+my_sort_func (gconstpointer a, gconstpointer b)
+{
+	gchar *sa, *sb;
+	sa = * (gchar**) a;
+	sb = * (gchar**) b;
+	if (sa && sb)
+		return g_utf8_collate (sa, sb);
+	else if (sa)
+		return -1;
+	else if (sb)
+		return 1;
+	else
+		return 0;
+}
+
 static gchar **
 make_array_from_strv (char **values, guint *out_size)
 {
@@ -739,13 +753,15 @@ make_array_from_strv (char **values, guint *out_size)
 	if (out_size)
 		*out_size = array->len;
 
+	g_array_sort (array, (GCompareFunc) my_sort_func);
+
 	return (gchar**) g_array_free (array, FALSE);
 }
 
 static gint
 classes_sort (GdaLdapClass *lcl1, GdaLdapClass *lcl2)
 {
-	return g_ascii_strcasecmp (lcl1->names[0], lcl2->names[0]);
+	return g_utf8_collate (lcl1->names[0], lcl2->names[0]);
 }
 
 static void
@@ -800,13 +816,13 @@ gda_ldap_get_g_type (LdapConnectionData *cdata, const gchar *attribute_name, con
 	GType coltype = GDA_TYPE_NULL;
 	if (specified_gtype)
 		coltype = gda_g_type_from_string (specified_gtype);
-	if (coltype == GDA_TYPE_NULL) {
+	if ((coltype == G_TYPE_INVALID) || (coltype == GDA_TYPE_NULL)) {
 		LdapAttribute *lat;
 		lat = gda_ldap_get_attr_info (cdata, attribute_name);
 		if (lat)
 			coltype = lat->type->gtype;
 	}
-	if (coltype == GDA_TYPE_NULL)
+	if ((coltype == G_TYPE_INVALID) || (coltype == GDA_TYPE_NULL))
 		coltype = G_TYPE_STRING;
 	return coltype;
 }
@@ -849,7 +865,22 @@ gda_ldap_attr_value_to_g_value (LdapConnectionData *cdata, GType type, BerValue
 		}
 		if (conv) {
 			struct tm *ptm;
+#ifdef HAVE_LOCALTIME_R
+			struct tm tmpstm;
+			ptm = localtime_r (&(tv.tv_sec), &tmpstm);
+#elif HAVE_LOCALTIME_S
+			struct tm tmpstm;
+			if (localtime_s (&tmpstm, &(tv.tv_sec)) == 0)
+				ptm = &tmpstm;
+			else
+				ptm = NULL;
+#else
 			ptm = localtime (&(tv.tv_sec));
+#endif
+
+			if (!ptm)
+				return NULL;
+
 			if (type == GDA_TYPE_TIMESTAMP) {
 				GdaTimestamp ts;
 				ts.year = ptm->tm_year + 1900;
