@@ -78,7 +78,7 @@ static void
 execution_statement_free (ExecutionStatement *estmt)
 {
 	if (estmt->exec_id > 0) {
-		/* job started => set up idle task */
+		/* job started! */
 		TO_IMPLEMENT;
 	}
 	else {
@@ -118,7 +118,6 @@ struct _QueryConsolePagePrivate {
 	GtkWidget *query_result;
 
 	ExecutionBatch *current_exec;
-	guint current_exec_id; /* timout ID to fetch execution results */
 
 	gint fav_id;
 	GtkWidget *favorites_menu;
@@ -197,8 +196,6 @@ query_console_page_dispose (GObject *object)
 
 	/* free memory */
 	if (tconsole->priv) {
-		if (tconsole->priv->current_exec_id)
-			g_source_remove (tconsole->priv->current_exec_id);
 		if (tconsole->priv->current_exec)
 			execution_batch_free (tconsole->priv->current_exec);
 		if (tconsole->priv->bcnc) {
@@ -268,6 +265,9 @@ static void sql_favorite_clicked_cb (GtkButton *button, QueryConsolePage *tconso
 static void history_copy_clicked_cb (GtkButton *button, QueryConsolePage *tconsole);
 static void history_clear_clicked_cb (GtkButton *button, QueryConsolePage *tconsole);
 static void history_changed_cb (QueryEditor *history, QueryConsolePage *tconsole);
+
+static void rerun_requested_cb (QueryResult *qres, QueryEditorHistoryBatch *batch,
+				QueryEditorHistoryItem *item, QueryConsolePage *tconsole);
 /**
  * query_console_page_new
  *
@@ -310,7 +310,7 @@ query_console_page_new (BrowserConnection *bcnc)
 	gtk_box_pack_start (GTK_BOX (hbox), hpaned, TRUE, TRUE, 0);
 
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
-	gtk_paned_pack1 (GTK_PANED (hpaned), vbox, TRUE, FALSE);
+	gtk_paned_pack1 (GTK_PANED (hpaned), vbox, TRUE, TRUE);
 
 	wid = gtk_label_new ("");
 	str = g_strdup_printf ("<b>%s</b>", _("SQL code to execute:"));
@@ -327,11 +327,10 @@ query_console_page_new (BrowserConnection *bcnc)
 			  G_CALLBACK (editor_changed_cb), tconsole);
 	g_signal_connect (wid, "execute-request",
 			  G_CALLBACK (editor_execute_request_cb), tconsole);
-	gtk_widget_set_size_request (wid, -1, 200);
 	
 	vbox = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	tconsole->priv->params_top = vbox;
-	gtk_paned_pack2 (GTK_PANED (hpaned), vbox, FALSE, TRUE);
+	gtk_paned_pack2 (GTK_PANED (hpaned), vbox, TRUE, TRUE);
 	
 	wid = gtk_label_new ("");
 	str = g_strdup_printf ("<b>%s</b>", _("Variables' values:"));
@@ -452,6 +451,8 @@ query_console_page_new (BrowserConnection *bcnc)
 	wid = query_result_new (tconsole->priv->history);
 	tconsole->priv->query_result = wid;
 	gtk_box_pack_start (GTK_BOX (vbox), wid, TRUE, TRUE, 0);
+	g_signal_connect (wid, "rerun-requested",
+			  G_CALLBACK (rerun_requested_cb), tconsole);
 
 	/* show everything */
         gtk_widget_show_all (vpaned);
@@ -489,9 +490,9 @@ history_changed_cb (G_GNUC_UNUSED QueryEditor *history, QueryConsolePage *tconso
 	QueryEditor *qe;
 	
 	qe = tconsole->priv->history;
-        if (query_editor_get_current_history_item (qe)) {
+        if (query_editor_get_current_history_item (qe, NULL)) {
 		query_result_show_history_item (QUERY_RESULT (tconsole->priv->query_result),
-						query_editor_get_current_history_item (qe));
+						query_editor_get_current_history_item (qe, NULL));
 		act = TRUE;
 	}
 	else if (query_editor_get_current_history_batch (qe)) {
@@ -521,7 +522,7 @@ history_copy_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsolePage *tcon
 
 	string = g_string_new ("");
 	qe = tconsole->priv->history;
-        qih = query_editor_get_current_history_item (qe);
+        qih = query_editor_get_current_history_item (qe, NULL);
         if (qih)
 		g_string_append (string, qih->sql);
         else {
@@ -871,15 +872,12 @@ params_form_changed_cb (GdauiBasicForm *form, G_GNUC_UNUSED GdaHolder *param,
 				  gdaui_basic_form_is_valid (form));
 }
 
-static gboolean query_exec_fetch_cb (QueryConsolePage *tconsole);
-
+static void actually_execute (QueryConsolePage *tconsole, const gchar *sql, GdaSet *params,
+			      gboolean add_editor_history);
 static void
 sql_execute_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsolePage *tconsole)
 {
 	gchar *sql;
-	const gchar *remain;
-	GdaBatch *batch;
-	GError *error = NULL;
 
 	/* compute parameters if necessary */
 	if (tconsole->priv->params_compute_id > 0) {
@@ -951,20 +949,144 @@ sql_execute_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsolePage *tcons
 		}
 	}
 
+	sql = query_editor_get_all_text (tconsole->priv->editor);
+	actually_execute (tconsole, sql, tconsole->priv->params, TRUE);
+	g_free (sql);
+}
+
+static void
+query_exec_done_cb (BrowserConnection *bcnc, guint exec_id,
+		    GObject *out_result, GdaSet *out_last_inserted_row,
+		    GError *error, QueryConsolePage *tconsole)
+{
+	gboolean alldone = TRUE;
+	
+	if (! tconsole->priv->current_exec)
+		return;
+
+	if (! tconsole->priv->current_exec->statements) {
+		execution_batch_free (tconsole->priv->current_exec);
+		tconsole->priv->current_exec = NULL;
+		return;
+	}
+
+	ExecutionStatement *estmt;
+	estmt = (ExecutionStatement*) tconsole->priv->current_exec->statements->data;
+	g_assert (estmt->exec_id == exec_id);
+	g_assert (!estmt->result);
+	g_assert (!estmt->exec_error);
+
+	ExecutionBatch *ebatch;
+	ebatch = tconsole->priv->current_exec;
+	query_editor_start_history_batch (tconsole->priv->history, ebatch->hist_batch);
+	if (out_result)
+		estmt->result = g_object_ref (out_result);
+	if (error)
+		estmt->exec_error = g_error_copy (error);
+
+	QueryEditorHistoryItem *history;
+	GdaSqlStatement *sqlst;
+	g_object_get (G_OBJECT (estmt->stmt), "structure", &sqlst, NULL);
+	if (!sqlst->sql) {
+		gchar *sql;
+		sql = gda_statement_to_sql (GDA_STATEMENT (estmt->stmt), NULL, NULL);
+		history = query_editor_history_item_new (sql, estmt->result, estmt->exec_error);
+		g_free (sql);
+	}
+	else
+		history = query_editor_history_item_new (sqlst->sql,
+							 estmt->result, estmt->exec_error);
+	gda_sql_statement_free (sqlst);
+
+	history->within_transaction = estmt->within_transaction;
+
+	/* display a message if a transaction has been started */
+	if (! history->within_transaction &&
+	    browser_connection_get_transaction_status (tconsole->priv->bcnc) &&
+	    gda_statement_get_statement_type (estmt->stmt) != GDA_SQL_STATEMENT_BEGIN) {
+		browser_window_show_notice_printf (BROWSER_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
+						   GTK_MESSAGE_INFO,
+						   "QueryExecTransactionStarted",
+						   "%s", _("A transaction has automatically been started\n"
+							   "during this statement's execution, this usually\n"
+							   "happens when blobs are selected (and the transaction\n"
+							   "will have to remain opened while the blobs are still\n"
+							   "accessible, clear the corresponding history item before\n"
+							   "closing the transaction)."));
+	}
+
+	query_editor_add_history_item (tconsole->priv->history, history);
+	query_editor_history_item_unref (history);
+
+	if (estmt->exec_error)
+		browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
+				    _("Error executing query:\n%s"),
+				    estmt->exec_error && estmt->exec_error->message ?
+				    estmt->exec_error->message : _("No detail"));
+	else
+		browser_window_push_status (BROWSER_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
+					    "QueryConsolePage", _("Statement executed"), TRUE);
+
+	/* get rid of estmt */
+	estmt->exec_id = 0;
+	execution_statement_free (estmt);
+	tconsole->priv->current_exec->statements =
+		g_slist_delete_link (tconsole->priv->current_exec->statements,
+				     tconsole->priv->current_exec->statements);
+	if (error)
+		goto onerror;
+
+	/* more to do ? */
+	if (tconsole->priv->current_exec->statements) {
+		estmt = (ExecutionStatement*) tconsole->priv->current_exec->statements->data;
+		estmt->within_transaction =
+			browser_connection_get_transaction_status (tconsole->priv->bcnc) ? TRUE : FALSE;
+		estmt->exec_id = browser_connection_execute_statement_cb (tconsole->priv->bcnc,
+									  estmt->stmt,
+									  tconsole->priv->params,
+									  GDA_STATEMENT_MODEL_RANDOM_ACCESS,
+									  FALSE,
+									  (BrowserConnectionExecuteCallback) query_exec_done_cb,
+									  tconsole, &(estmt->exec_error));
+		if (estmt->exec_id == 0) {
+			browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
+					    _("Error executing query:\n%s"),
+					    estmt->exec_error && estmt->exec_error->message ?
+					    estmt->exec_error->message : _("No detail"));
+			goto onerror;
+		}
+	}
+	
+	if (! tconsole->priv->current_exec->statements) {
+		execution_batch_free (tconsole->priv->current_exec);
+		tconsole->priv->current_exec = NULL;
+	}
+	return;
+
+ onerror:
+	execution_batch_free (tconsole->priv->current_exec);
+	tconsole->priv->current_exec = NULL;
+}
+
+static void
+actually_execute (QueryConsolePage *tconsole, const gchar *sql, GdaSet *params,
+		  gboolean add_editor_history)
+{
+	GdaBatch *batch;
+	GError *error = NULL;
+	const gchar *remain;
+
 	if (!tconsole->priv->parser)
 		tconsole->priv->parser = browser_connection_create_parser (tconsole->priv->bcnc);
 
-	sql = query_editor_get_all_text (tconsole->priv->editor);
 	batch = gda_sql_parser_parse_string_as_batch (tconsole->priv->parser, sql, &remain, &error);
 	if (!batch) {
 		browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
 				    _("Error while parsing code: %s"),
 				    error && error->message ? error->message : _("No detail"));
 		g_clear_error (&error);
-		g_free (sql);
 		return;
 	}
-	g_free (sql);
 
 	/* if a query is being executed, then show an error */
 	if (tconsole->priv->current_exec) {
@@ -975,8 +1097,10 @@ sql_execute_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsolePage *tcons
 		return;
 	}
 
-	/* mark the current SQL to be kept by the editor as an internal history */
-	query_editor_keep_current_state (tconsole->priv->editor);
+	if (add_editor_history) {
+		/* mark the current SQL to be kept by the editor as an internal history */
+		query_editor_keep_current_state (tconsole->priv->editor);
+	}
 
 	/* actual Execution */
 	const GSList *stmt_list, *list;
@@ -984,7 +1108,7 @@ sql_execute_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsolePage *tcons
 	ebatch = g_new0 (ExecutionBatch, 1);
 	ebatch->batch = batch;
 	g_get_current_time (&(ebatch->start_time));
-	ebatch->hist_batch = query_editor_history_batch_new (ebatch->start_time, tconsole->priv->params);
+	ebatch->hist_batch = query_editor_history_batch_new (ebatch->start_time, params);
 
 	stmt_list = gda_batch_get_statements (batch);
 	for (list = stmt_list; list; list = list->next) {
@@ -993,145 +1117,49 @@ sql_execute_clicked_cb (G_GNUC_UNUSED GtkButton *button, QueryConsolePage *tcons
 		estmt->stmt = GDA_STATEMENT (list->data);
 		ebatch->statements = g_slist_prepend (ebatch->statements, estmt);
 
-		if (list == stmt_list) {
-			estmt->within_transaction =
-				browser_connection_get_transaction_status (tconsole->priv->bcnc) ? TRUE : FALSE;
-			estmt->exec_id = browser_connection_execute_statement (tconsole->priv->bcnc,
-									       estmt->stmt,
-									       tconsole->priv->params,
-									       GDA_STATEMENT_MODEL_RANDOM_ACCESS,
-									       FALSE, &(estmt->exec_error));
-			if (estmt->exec_id == 0) {
-				browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
-						    _("Error executing query: %s"),
-						    estmt->exec_error && estmt->exec_error->message ? estmt->exec_error->message : _("No detail"));
-				execution_batch_free (ebatch);
-				return;
-			}
+		if (list != stmt_list)
+			continue;
+
+		/* only the 1st statement is actually executed, to be able to
+		 * stop others following statements to be executed if there is an
+		 * execution error.
+		 */
+		estmt->within_transaction =
+			browser_connection_get_transaction_status (tconsole->priv->bcnc) ? TRUE : FALSE;
+		estmt->exec_id = browser_connection_execute_statement_cb (tconsole->priv->bcnc,
+									  estmt->stmt,
+									  params,
+									  GDA_STATEMENT_MODEL_RANDOM_ACCESS,
+									  FALSE,
+									  (BrowserConnectionExecuteCallback) query_exec_done_cb,
+									  tconsole, &(estmt->exec_error));
+		if (estmt->exec_id == 0) {
+			browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
+					    _("Error executing query: %s"),
+					    estmt->exec_error && estmt->exec_error->message ? estmt->exec_error->message : _("No detail"));
+			execution_batch_free (ebatch);
+			return;
 		}
 	}
 	ebatch->statements = g_slist_reverse (ebatch->statements);
-
 	tconsole->priv->current_exec = ebatch;
-	tconsole->priv->current_exec_id = g_timeout_add (200,
-							 (GSourceFunc) query_exec_fetch_cb,
-							 tconsole);
 }
 
-static gboolean
-query_exec_fetch_cb (QueryConsolePage *tconsole)
+static void
+rerun_requested_cb (QueryResult *qres, QueryEditorHistoryBatch *batch,
+		    QueryEditorHistoryItem *item, QueryConsolePage *tconsole)
 {
-	gboolean alldone = TRUE;
-	
-	if (tconsole->priv->current_exec) {
-		if (tconsole->priv->current_exec->statements) {
-			ExecutionStatement *estmt;
-			estmt = (ExecutionStatement*) tconsole->priv->current_exec->statements->data;
-			g_assert (estmt->exec_id);
-			g_assert (!estmt->result);
-			g_assert (!estmt->exec_error);
-			estmt->result = browser_connection_execution_get_result (tconsole->priv->bcnc,
-										 estmt->exec_id, NULL,
-										 &(estmt->exec_error));
-			if (estmt->result || estmt->exec_error) {
-				ExecutionBatch *ebatch;
-				ebatch = tconsole->priv->current_exec;
-				query_editor_start_history_batch (tconsole->priv->history, ebatch->hist_batch);
-
-				QueryEditorHistoryItem *history;
-				GdaSqlStatement *sqlst;
-				g_object_get (G_OBJECT (estmt->stmt), "structure", &sqlst, NULL);
-				if (!sqlst->sql) {
-					gchar *sql;
-					sql = gda_statement_to_sql (GDA_STATEMENT (estmt->stmt), NULL, NULL);
-					history = query_editor_history_item_new (sql, estmt->result, estmt->exec_error);
-					g_free (sql);
-				}
-				else
-					history = query_editor_history_item_new (sqlst->sql,
-										 estmt->result, estmt->exec_error);
-				gda_sql_statement_free (sqlst);
-
-				history->within_transaction = estmt->within_transaction;
-
-				if (estmt->exec_error) {
-					browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
-							    _("Error executing query:\n%s"),
-							    estmt->exec_error && estmt->exec_error->message ?
-							    estmt->exec_error->message : _("No detail"));
-					TO_IMPLEMENT;
-					/* FIXME: offer the opportunity to stop execution of batch if there
-					 *        are some remaining statements to execute
-					 * FIXME: store error in history => modify QueryEditorHistoryItem
-					 */
-				}
-				else
-					browser_window_push_status (BROWSER_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
-								    "QueryConsolePage", _("Statement executed"), TRUE);
-
-				/* display a message if a transaction has been started */
-				if (! history->within_transaction &&
-				    browser_connection_get_transaction_status (tconsole->priv->bcnc) &&
-				    gda_statement_get_statement_type (estmt->stmt) != GDA_SQL_STATEMENT_BEGIN) {
-					browser_window_show_notice_printf (BROWSER_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
-									   GTK_MESSAGE_INFO,
-									   "QueryExecTransactionStarted",
-									   "%s", _("A transaction has automatically been started\n"
-										   "during this statement's execution, this usually\n"
-										   "happens when blobs are selected (and the transaction\n"
-										   "will have to remain opened while the blobs are still\n"
-										   "accessible, clear the corresponding history item before\n"
-										   "closing the transaction)."));
-				}
-				
-				query_editor_add_history_item (tconsole->priv->history, history);
-				query_editor_history_item_unref (history);
-
-				/* get rid of estmt */
-				estmt->exec_id = 0;
-				execution_statement_free (estmt);
-				tconsole->priv->current_exec->statements =
-					g_slist_delete_link (tconsole->priv->current_exec->statements,
-							     tconsole->priv->current_exec->statements);
-
-				/* more to do ? */
-				if (tconsole->priv->current_exec->statements) {
-					estmt = (ExecutionStatement*) tconsole->priv->current_exec->statements->data;
-					estmt->within_transaction =
-						browser_connection_get_transaction_status (tconsole->priv->bcnc) ? TRUE : FALSE;
-					estmt->exec_id = browser_connection_execute_statement (tconsole->priv->bcnc,
-											       estmt->stmt,
-											       tconsole->priv->params,
-											       GDA_STATEMENT_MODEL_RANDOM_ACCESS,
-											       FALSE,
-											       &(estmt->exec_error));
-					if (estmt->exec_id == 0) {
-						browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
-								    _("Error executing query:\n%s"),
-								    estmt->exec_error && estmt->exec_error->message ?
-								    estmt->exec_error->message : _("No detail"));
-					}
-					else
-						alldone = FALSE;
-				}
-			}
-			else
-				alldone = FALSE;
-		}
-	}
-	
-	if (alldone) {
-		if (tconsole->priv->current_exec) {
-			execution_batch_free (tconsole->priv->current_exec);
-			tconsole->priv->current_exec = NULL;
-		}
-
-		tconsole->priv->current_exec_id = 0;
+	if (!batch || !item || !item->sql) {
+		browser_show_error (GTK_WINDOW (gtk_widget_get_toplevel ((GtkWidget*) tconsole)),
+				    _("Internal error, please report error to "
+				      "http://bugzilla.gnome.org/ for the \"libgda\" product"));
+		return;
 	}
 
-	return !alldone;
+	actually_execute (tconsole, item->sql, batch->params, FALSE);
 }
 
+
 /**
  * query_console_page_set_text
  * @console: a #QueryConsolePage
